@@ -82,7 +82,7 @@ else  { coordx=estobj$coordx;
 ##################################
 
 if(is.null(CkModel(model))) stop("The name of the  model  is not correct\n")
-if(is.null(CkCorrModel (corrmodel))) stop("The name of the correlation model  is not correct\n")
+    if( !is.character(corrmodel)|| is.null(CkCorrModel(corrmodel)))       stop("the name of the correlation model is wrong")
 corrmodel=gsub("[[:blank:]]", "",corrmodel); model=gsub("[[:blank:]]", "",model)
 distance=gsub("[[:blank:]]", "",distance); method=gsub("[[:blank:]]", "",method)
 type_krig=gsub("[[:blank:]]", "",type_krig); type=gsub("[[:blank:]]", "",type)
